// Crosswalk の場合、serif と sans-serif 両方が指定されていると正しい font が選択されないので最初に指定されたもの以外を削除する。
function change_default_font(rule){
    if(rule.style.fontFamily){
        var fonts = rule.style.fontFamily.split(",");
        var new_font_family = "";
        var isSerifOrSansSerif = false;
        var santen_font_name;
        for(var k=0; k<fonts.length; k++){
            var font_name = fonts[k];
            
            //先頭と最後尾の空白を削除
            font_name = font_name.trim();
            
            if(k == 0){
                new_font_family = font_name;
                if (font_name === "serif" || font_name === "sans-serif") {
                    isSerifOrSansSerif= true;
                    santen_font_name = (font_name === "serif")? "serif-santen": "sans-serif-santen";
                }
            }
            else{
                if (font_name === "serif" || font_name === "sans-serif") {
                    if (isSerifOrSansSerif === true) {
                        continue;
                    }
                    isSerifOrSansSerif= true;
                    santen_font_name = (font_name === "serif")? "serif-santen": "sans-serif-santen";
                }
                new_font_family = new_font_family + ", " + font_name;
            }
        }

        if (santen_font_name) {
            rule.style.fontFamily = santen_font_name + ", " + new_font_family;
        }
        else {
            rule.style.fontFamily = new_font_family;
        }
    }
}

function change_page_break(rule){
    if(rule.style.pageBreakBefore){
        rule.style.webkitColumnBreakBefore = rule.style.pageBreakBefore;
    }
    if(rule.style.pageBreakAfter){
        rule.style.webkitColumnBreakAfter = rule.style.pageBreakAfter;
    }
}

function change_stylesheet_Android(stylesheet){
    var rules = stylesheet.cssRules;
    
    if(rules){
        for(var j = 0; j < rules.length; j++){
            var rule = rules[j];
            
            if(rule.type == CSSRule.IMPORT_RULE){
                change_stylesheet_Android(rule.styleSheet);
            }
            if(rule.type == CSSRule.STYLE_RULE){
                change_default_font(rule);
                change_page_break(rule);
            }
        }
    }
}

function change_stylesheets_Android(doc){
    for(var i = 0 ; i < doc.styleSheets.length; i++) {
        var styleSheet = doc.styleSheets[i];
        change_stylesheet_Android(styleSheet);
    }
}

function check_and_modify_media_property(elem, ppty) {
    if (elem.hasAttribute(ppty)) {
        if (ppty !== 'controls') {
            elem.removeAttribute(ppty);
        }
        elem.setAttribute('reader_'+ppty, 'true');
    }
}

function check_and_modify_medias_property(doc) {
    var mediaTags = doc.querySelectorAll('audio');
    var loopIndex, elem, ppty;
    if (mediaTags && mediaTags.length > 0) {
        for (loopIndex = 0; loopIndex < mediaTags.length; loopIndex++) {
            elem = mediaTags.item(loopIndex);
            elem.setAttribute('preload', 'none');
            check_and_modify_media_property(elem, 'autoplay');
            check_and_modify_media_property(elem, 'loop');
            check_and_modify_media_property(elem, 'controls');
        }
    }
    var embedTags = doc.querySelectorAll('object, iframe, embed');
    if (embedTags && embedTags.length > 0) {
        for (loopIndex = 0; loopIndex < embedTags.length; loopIndex++) {
            elem = embedTags.item(loopIndex);
            if (elem.contentDocument) {
                check_and_modify_medias_property(elem.contentDocument);
            }
        }
    }

}

document.addEventListener("DOMContentLoaded", function(event) {
    change_stylesheets_Android(document);
    check_and_modify_medias_property(document);
});
