//for detect platform.
//from helpers.js
Platform = {
    /// @function ReadiumSDK.Helpers.Platform.getPlatform
    /// return platform as String. see osPlatformEnum.
    getPlatform: function () { this.initialize(); return this.osPlatform;},
    
    /// @function ReadiumSDK.Helpers.Platform.getOsVersionString()
    /// return version number string, such as "4.4.2" or "8_1_2".
    getOsVersionString: function () {this.initialize(); return this.osVersionString;},
    
    /// @function ReadiumSDK.Helpers.Platform.getOsVersionMajorNumber()
    /// return major number of version as Int.
    getOsVersionMajorNumber: function () {this.initialize(); return this.osVersionMajorNumber;},
    
    /// @function ReadiumSDK.Helpers.Platform.getOsVersionMinorNumber()
    /// return minor number of version as Int.
    getOsVersionMinorNumber: function () {this.initialize(); return this.osVersionMinorNumber;},
    
    /// @function ReadiumSDK.Helpers.Platform.getOsVersionRevisionNumber()
    /// return revision number of version as Int.
    getOsVersionRevisionNumber: function () {this.initialize(); return this.osVersionRevisionNumber;},
    
    /// @function ReadiumSDK.Helpers.Platform.isOsIOS()
    /// return this platform is iOS or not as boolean.
    isOsIOS: function () {return this.getPlatform() === this.osPlatformEnum.IOS;},
    
    /// @function ReadiumSDK.Helpers.Platform.isOsAndroid()
    /// return this platform is Android or not as boolean.
    isOsAndroid: function () {return this.getPlatform() === this.osPlatformEnum.ANDROID;},
    
    /// @function ReadiumSDK.Helpers.Platform.isOsLinux()
    /// return this platform is Linux or not as boolean.
    isOsLinux: function () {return this.getPlatform() === this.osPlatformEnum.LINUX;},
    
    osPlatformEnum: {UNKNOWN:"unknown", IOS: "iOS", ANDROID: "android", LINUX: "linux"},
    webViewEnum: {UNKNOWN: "unknown", EMBED: "embed", XWALK: "crosswalk"},
    
    userAgent: "",
    osPlatform: "",
    osVersionString: "",
    osVersionMajorNumber: 0,
    osVersionMinorNumber: 0,
    osVersionRevisionNumber: 0,
    webView: "",
    
    initialize: function () {
        if (this.userAgent !== "") return;
        var ua = window.navigator.userAgent;
        var pf = navigator.platform;
        this.userAgent = ua;
        this.osPlatform = this.osPlatformEnum.UNKNOWN;
        this.webView = this.webViewEnum.EMBED;
        
        if (/iP(hone|od|ad)/.test(pf)) {// this is iOS.
            this.osPlatform = this.osPlatformEnum.IOS;
            this.webView = this.webViewEnum.EMBED;
            
            var verString = ua.match(/OS (\d+)_(\d+)_?(\d+)?/);
            if (verString === null) return;
            
            this.osVersionString = verString[0].substring("OS ".length);
            this.osVersionMajorNumber = parseInt(verString[1], 10);
            this.osVersionMinorNumber = parseInt(verString[2], 10);
            this.osVersionRevisionNumber = parseInt(verString[3], 10);
        }
        else if (/ndroid/.test(ua)) {	// this is android.
            this.osPlatform = this.osPlatformEnum.ANDROID;
            this.webView = this.webViewEnum.EMBED;
        
            verString = ua.match(/ndroid (\d+)\.(\d+)\.?(\d+)?/);
            if (verString === null) return;
        
            this.osVersionString = verString[0].substring("ndroid ".length);
            this.osVersionMajorNumber = parseInt(verString[1], 10);
            this.osVersionMinorNumber = parseInt(verString[2], 10);
            this.osVersionRevisionNumber = parseInt(verString[3], 10);
        
            if (/rosswalk/.test(ua)) {
                this.webView = this.webViewEnum.XWALK;
            }
        }
        else if (/inux/.test(ua)) {	// this is linux.
            this.osPlatform = this.osPlatformEnum.LINUX;
        }
        else {  // unknown device
            this.webView = this.webViewEnum.UNKNOWN;
        }
    }
};
