function getUrlQueryList(){
    var list = [], max = 0, hash = "", array = "";
    var url = window.location.search;
    
    hash  = url.slice(1).split('&');
    max = hash.length;
    for (var i = 0; i < max; i++) {
        array = hash[i].split('=');
        list.push(array[0]);
        list[array[0]] = decodeURIComponent(array[1]);
    }
    
    return list;
}

function addStyleSheetIntoHtmlHead(html){
    var baseURL = window.location.origin + window.location.pathname + '/../toc/content_css/';
    
    if(Platform.isOsIOS()){
        html = addStringIntoHtmlHeadTop(html, '<link href=\"' + baseURL + 'toc_default_iOS.css\" rel=\"stylesheet\" type=\"text/css\" />');
        html = addStringIntoHtmlHeadBottom(html, '<link href=\"' + baseURL + 'toc_override_iOS.css\" rel=\"stylesheet\" type=\"text/css\" />');
    }
    else if(Platform.isOsAndroid()){
        html = addStringIntoHtmlHeadTop(html, '<link href=\"' + baseURL + 'toc_default_Android.css\" rel=\"stylesheet\" type=\"text/css\" />');
        html = addStringIntoHtmlHeadBottom(html, '<link href=\"' + baseURL + 'toc_override_Android.css\" rel=\"stylesheet\" type=\"text/css\" />');
    }
    
    return html;
}

function addScriptIntoHtmlHead(html){
    var baseURL = window.location.origin + window.location.pathname + '/../toc/content_js/';
    
    html = addStringIntoHtmlHeadTop(html, '<script src=\"' + baseURL + '../../lib/thirdparty/jquery-2.1.3.min.js\" type=\"text/javascript\"></script>');
    html = addStringIntoHtmlHeadBottom(html, '<script src=\"' + baseURL + '../platform.js\" type=\"text/javascript\"></script>');
    html = addStringIntoHtmlHeadBottom(html, '<script src=\"' + baseURL + 'toc_additional_script.js\" type=\"text/javascript\"></script>');
    if(Platform.isOsIOS()){
        html = addStringIntoHtmlHeadBottom(html, content_additional_script_ios_string());
    }
    
    return html;
}

function addStringIntoHtmlHeadTop(html, string){
    html = html.replace(/(<head[^>]*>)/, '$1' + string);
    
    return html;
}

function addStringIntoHtmlHeadBottom(html, string){
    html = html.replace(/(<\/head[^>]*>)/, string + '$1');
    
    return html;
}

function addCssAndJsToToc(html){
    html = addStyleSheetIntoHtmlHead(html);
    html = addScriptIntoHtmlHead(html);
    
    return html;
}

function fetchHtmlAsText(path, callback){
    xmlHttp = new XMLHttpRequest();
    xmlHttp.onreadystatechange = function(result){
        if (xmlHttp.readyState == 4 && xmlHttp.status == 200){
            callback(xmlHttp.responseText);
        }
    };
    xmlHttp.open("GET", path, true);
    xmlHttp.send(null);
}

function fetchContentDocument(src, callback){
    fetchHtmlAsText(src, function (html){
        if (!html) {
            callback();
            return;
        }
        
        var sourceParts = src.split("/");
        sourceParts.pop(); //remove source file name
                    
        html = addCssAndJsToToc(html);
        html = addStringIntoHtmlHeadTop(html, '<base href=\"' + sourceParts.join('/') + '/' + '\"/>');
        if (Platform.isOsIOS()) {
            html = addStringIntoHtmlHeadTop(html, '<meta name="viewport" content="minimum-scale=1.0, initial-scale=1.0, maximum-scale=4.0">');
        }
        else {
            html = addStringIntoHtmlHeadTop(html, '<meta name="viewport" content="minimum-scale=1.0, initial-scale=1.0, maximum-scale=1.0">');
        }
                    
        callback(html);
    });
}

var defaultSansSerifSystemFontName = "SDR-TsukuGo Pr6N";
var defaultSerifSystemFontName = "SDR-TsukuMin Pr6N";

document.addEventListener("DOMContentLoaded", function(event){
    var queryList = getUrlQueryList();
    var tocURL = queryList["toc"];
    var fontSize = queryList["fontSize"];
    var defaultFontFaceJsonString = queryList["defaultFontFace"];
    var defaultFontFace;

    if (defaultFontFaceJsonString) {
        defaultFontFace = JSON.parse(defaultFontFaceJsonString);
        if (defaultFontFace.serif) {
            defaultSerifSystemFontName = defaultFontFace.serif;
        }
        if (defaultFontFace.sansSerif) {
            defaultSansSerifSystemFontName = defaultFontFace.sansSerif;
        }
    }
                          
    //load toc html as text
    fetchContentDocument(tocURL, function(html){
        if(!html){
            return;
        }
                
        //現在のhtmlをtocのhtmlで置き換える
        var blobTocURL = URL.createObjectURL(
            new Blob([html], {'type': 'text/html'})
        );
        window.location = blobTocURL;
        window.onload = function() {
            URL.revokeObjectURL(blobTocURL);
        };
    });
});
function content_additional_script_ios_string () {
return '<script type="text/javascript"> \n\
function change_stylesheets_for_default_font(doc){ \n\
    for (var i = 0 ; i < doc.styleSheets.length ; i++) { \n\
        var ss = doc.styleSheets[i]; \n\
        change_stylesheet_for_default_font(ss); \n\
    } \n\
} \n\
function change_stylesheet_for_default_font(stylesheet){ \n\
    var rules; \n\
    rules = stylesheet.cssRules; \n\
    if(rules){ \n\
        for (var j = 0 ; j < rules.length ; j++) { \n\
            var rule = rules[j]; \n\
            if(rule.type == CSSRule.IMPORT_RULE){ \n\
                this.change_stylesheet_for_default_font(rule.styleSheet); \n\
            } \n\
            else if(rule.type == CSSRule.STYLE_RULE){ \n\
                if(rule.style.fontFamily){ \n\
                    var fonts = rule.style.fontFamily.split(","); \n\
                    var new_font_family = ""; \n\
                    for(var k=0; k<fonts.length; k++){ \n\
                        var font_name = fonts[k]; \n\
                        font_name = font_name.trim(); \n\
                        if(font_name == "sans-serif"){ \n\
                            font_name = \'"' + defaultSansSerifSystemFontName + '"\'; \n\
                        } \n\
                        else if(font_name == "serif" || font_name == "monospace" || font_name == "cursive" || font_name == "fantasy"){ \n\
                            font_name = \'"' + defaultSerifSystemFontName + '"\'; \n\
                        } \n\
                        if(k == 0){ \n\
                            new_font_family = font_name; \n\
                        } \n\
                        else{ \n\
                            new_font_family = new_font_family + ", " + font_name; \n\
                        } \n\
                    } \n\
                    rule.style.fontFamily = new_font_family; \n\
                } \n\
            } \n\
        } \n\
    } \n\
} \n\
</script>';
}
