// Copyright 2016 Sony Marketing (Japan) Inc.

define(['jquery', 'URIjs', 'Readium', 'Spinner', 'storage/Settings', 'hgn!templates/reader-body.html'],
function ($, URI, Readium, spinner, Settings, ReaderBody) {

    var nonLinearView;

    function createView () {
        var defaultSettings = {};
        var $appContainer = $('#app-container');
        $appContainer.empty();
        $appContainer.append(ReaderBody({}));

        var initWidth = $("#epub-reader-frame").width();
        var initHeight = $("#epub-reader-frame").height();

        var iframeLoader = new ReadiumSDK.Views.IFrameLoader();
        var options =  {
            $viewport: $("#epub-reader-frame"),
            settings: defaultSettings,
            viewportWidth: initWidth,
            viewportHeight: initHeight,
            spine: null,
            iframeLoader: new ReadiumSDK.Views.IFrameLoader()
        };

        var readiumOptions = {
            jsLibRoot: '../lib/thirdparty/',
            openBookOptions: {}
        };

        nonLinearView = new ReadiumSDK.Views.NonLinearGriffinView(options);
        nonLinearView.render();
        nonLinearView.on(ReadiumSDK.Events.CONTENT_DOCUMENT_LOADED, function (linksInfo) {
            stopSpin();
            sendSonyReaderNotification("nonlinearLoaded", JSON.stringify({'tapArea': linksInfo}));
        });
        nonLinearView.on(ReadiumSDK.Events.INTERNAL_LINK_CLICKED, function (urlInfo) {
            if (urlInfo && urlInfo.href && urlInfo.type !== undefined && urlInfo.href !== "") {
                if (urlInfo.type !== undefined) {
                    sendSonyReaderNotification("internalLinkTappedNotification", JSON.stringify({"errorCode": 0, "internalLinkUrl": urlInfo.href, "type": urlInfo.type}));
                }
                else {
                    sendSonyReaderNotification("internalLinkTappedNotification", JSON.stringify({"errorCode": 0, "internalLinkUrl": urlInfo.href}));
                }
            }
        });
    };

    function openNonLinearSpine (data) {
        var nonlinear_url = data['url'];
        var pageOpenRequest = new ReadiumSDK.Models.PageOpenRequest(null, self);
        createView();
        nonLinearView.openPage(nonlinear_url);
        spin();
    }

    function sendSonyReaderNotification(funcName, paramString){
        var androidAlsoUseSchemaLogic = false;
        var sonyReaderSchema = "griffin-notification://";
        var pfm = ReadiumSDK.Helpers.Platform;
        var encodedParamString;
        var uriString;
        var $appContainer;

        if (paramString) {
            encodedParamString = encodeURIComponent(paramString);
        }
        if (pfm.isOsIOS() || androidAlsoUseSchemaLogic) {
            uriString = sonyReaderSchema + funcName;
            if (paramString) uriString += "?result=" + encodedParamString;

            $appContainer = $('#app-container');
            setTimeout(function () {
                var notificationListener = document.createElement('iframe');
                notificationListener.setAttribute('src', uriString);
                $appContainer[0].appendChild(notificationListener);
                $appContainer[0].removeChild(notificationListener);
                $appContainer = undefined;
            }, 0);
        }
        else if (pfm.isOsAndroid()) {
	    uriString = sonyReaderSchema + funcName;
	    if (paramString) uriString += "?result=" + encodedParamString;
            xWalkEventAcceptor.setResult(uriString);

        }
        else {
            return;
        }
        console.log("## sendSonyReaderNotification:" + sonyReaderSchema + funcName + "?result=" + paramString);
    }

    var spin = function()
    {
        if (spinner.willSpin || spinner.isSpinning) return;

        spinner.willSpin = true;

        setTimeout(function()
                   {
                       if (spinner.stopRequested) {
                           spinner.willSpin = false;
                           spinner.stopRequested = false;
                           return;
                       }
                       spinner.isSpinning = true;
                       spinner.spin($('#reading-area')[0]);

                       spinner.willSpin = false;

                   }, 0);
    };
    var stopSpin = function()
    {
        if (spinner.isSpinning) {
            spinner.stop();
            spinner.isSpinning = false;
        }
        else if (spinner.willSpin) {
            spinner.stopRequested = true;
        }
    };



    // make EbubReader object and return it.
    return {
        openNonLinearSpine: openNonLinearSpine
    };

});
